// udStFiLrMiniSoft
// J. J. Weimer
// History: $Revision: 878 $ on $Date: 2008-07-30 14:09:16 -0500 (Wed, 30 Jul 2008) $
//
// Procedure files to read minisoft files and store them
//
// Access to Package
//
// -> udStFiLrMiniSoft(udFL)
//		STRUCTURE udFLStandardStructure &udFL
//
// see the structure definition and User's Manual

#pragma rtGlobals=1
#pragma IgorVersion=6.03
#pragma version=1.1

#include "udFLStandardFunctions", versions>=2.0

// **********************************************
// Globals and Localizations

Static Constant kVersion = 1.0

// **********************************************
// Initialize (Zero + Query + Check for PlugIns)

Static Function udFLMiniSoftInititalize(udFL)
	STRUCT udFLStandardStructure &udFL
	
	udFLZeroStructure(udFL)
	udFLMiniSoftQuery(udFL)
	return 0
end

// **********************************************
// Query

Static Function udFLMiniSoftQuery(udFL)
	STRUCT udFLStandardStructure &udFL
		
	udFL.mimetype = "TEXT"
	udFL.extensions = ".el1;"
	udFL.loadModes += "Manual;"
	udFL.procModes += "Remove Scaling;"
	udFL.itemsType = 4			// returns waves
	udFL.itemsDim[p] = p*0
	udFL.itemsDim[0]=1			// waves are nominally scaled numeric
	udFL.setNames=(2+8)		// filename+number or prefix+number
	udFLMiniSoftInitPlugIns(udFL)
	return 0	
end

// **********************************************
// Check for PlugIns

Static Function udFLMiniSoftInitPlugIns(udFL)
	STRUCT udFLStandardStructure &udFL

//	udFLCheckForPlugIn("udFLPlugInLinkDisplay",udFL)
	return 0
end

// **********************************************
// File Loader

Function udStFiLrMiniSoft(udFL)
	STRUCT udFLStandardStructure &udFL

	variable refNum, ic, nt
	string pathStr, wavesLoaded, twList
	
	switch(udFL.userLCtrl)
		case -1:		// initialize		
			udFLMiniSoftInititalize(udFL)		
			return 0
		case 0:		// query
			udFLMiniSoftQuery(udFL)
			return 0
		case 1:		// auto
			if (strlen(udFL.fileList)==0)
				udFLSetError(udFL,"No files in the selection list!",-1)
				return -1
			endif
			break
		case 2:		// manual
			Open/D/Z=2/M="Looking for a MiniSoft file"/R/T=".el1" refnum
			if (strlen(S_filename)!=0)
				udFL.filelist = S_filename + ";"
			else
//				udFLSetError(udFL,"No file selected",-1)
				return -1
			endif
			break
	endswitch
	
	if (udFL.reportCtrl >= 1)
		print "LOADING MINI-SOFT DATA"
		print "... assuring file paths"
	endif
	if (udFLAssureFilePaths(udFL)<0)
		return -1
	endif
	if (strlen(udFL.pathStr)!=0)
		pathStr = udFL.pathStr
		NewPath/O/Q tmpMiniSoftdir, pathStr
	endif
	
	if (udFL.reportCtrl >= 1)
		print "... setting data folder location"
	endif
	string cdf = GetDataFolder(1)
	udFLSetDataFolder(udFL)
	
	if (udFL.reportCtrl >= 1)
		print "... loading"
	endif
		
	nt = ItemsInList(udFL.fileList)
	for (ic=0;ic<nt;ic+=1)
			
		WavesLoaded = LoadMiniSoftFile(ic,udFL)
		
		if (strlen(WavesLoaded)!=0)
				
			WavesLoaded = ProcessMiniSoftWaves(WavesLoaded, udFL)
			WavesLoaded = RenameMiniSoftWaves(ic,WavesLoaded, udFL)
//			WavesLoaded = DoUserDataMiniSoftWaves(WavesLoaded, udFL)
//			LinkMiniSoftWaves(WavesLoaded,udFL)
			udFL.itLst.WAVES += WavesLoaded
				
		endif
		
	endfor
	
	udFL.itemsCount = ItemsInList(udFL.itLst.WAVES)
	
	SetDataFolder $cdf
	
	if (strlen(udFL.pathStr)!=0)
		KillPath tmpMiniSoftDir
	endif
	
	if (udFL.reportCtrl == 2)
		print "Structure @ Loader:", udFL
	endif
	
	return 0
end

// **********************************************
// Load A File

Static Function/S LoadMiniSoftFile(ic, udFL)
	variable ic
	STRUCT udFLStandardStructure &udFL

	string fileStr=StringFromList(ic,udFL.fileList)
	variable refNum
	
	string noteStr=""
	
	if (udFL.reportCtrl >= 1)
		print "... file ", fileStr
	endif
	if (strlen(udFL.pathStr)==0)
		Open/r refNum fileStr
	else
		Open/r/P=tmpMiniSoftdir refNum fileStr
	endif

	string S_fileNameLocal="MiniSoftFile"
	variable V_numDataPnts,V_timeDelta,V_timeStart,V_startComment,V_commentLen
	variable V_startWindowTitle,V_windowTitleLen,V_tmp,V_measureMin,V_measureMax
	variable V_version,V_alarmMin,V_alarmMax,V_alarmDelay,V_degree
	string S_comment,S_windowTitle,S_identification
	fsetpos refnum,99; fbinread/b=3/f=3 refnum,V_startComment
	V_startComment-=1	// in Igor, first byte is byte #0!!!
	fsetpos refnum,1007; fbinread/b=3/f=5 refnum,V_version
	fsetpos refnum,1111; fbinread/b=3/f=5 refnum,V_measureMin
	fsetpos refnum,1119; fbinread/b=3/f=5 refnum,V_measureMax
	fsetpos refnum,1503; fbinread/b=3/f=5 refnum,V_alarmMin
	fsetpos refnum,1511; fbinread/b=3/f=5 refnum,V_alarmMax
	fsetpos refnum,1519; fbinread/b=3/f=3 refnum,V_alarmDelay
	fsetpos refnum,2003; fbinread/b=3/f=3 refnum,V_numDataPnts
	fsetpos refnum,2011; fbinread/b=3/f=3 refnum,V_timeDelta
	fsetpos refnum,3999; fbinread/b=3/f=5 refnum,V_timeStart
	V_timeStart=V_timeStart*24*60*60-126313200	// Conversion into s, offset subtraction
	make/o/n=(V_numDataPnts) $S_filenameLocal
	wave w=$S_filenameLocal
	fsetpos refnum,4007; fbinread/b=3/f=5 refnum,w	// data start @ 4007
	setscale/p x V_timeStart,V_timeDelta,"dat" w
	V_degree= (cmpstr(IgorInfo(2),"Macintosh")==0) ? 251 : 176	// mac + win compatible
	setscale d 0,0,(num2char(V_degree)+"C") w
	S_identification=padstring("",39,0x20)
	fsetpos refnum,2103; fbinread refnum,S_identification
	S_identification=replacestring(num2char(32),S_identification,"")
	fsetpos refnum,V_startComment
	fstatus refnum
	do
		if(V_filePos==V_logEOF)
			break
		endif
		fbinread/f=1 refnum,V_tmp; fstatus refnum
	while(V_tmp!=4)
	V_commentLen=V_filePos-V_startComment-1
	do
		if(V_filePos==V_logEOF)
			break
		endif
		fbinread/f=1 refnum,V_tmp; fstatus refnum
	while(V_tmp!=0)
	V_startWindowTitle=V_filePos
	do
		if(V_filePos==V_logEOF)
			break
		endif
		fbinread/f=1 refnum,V_tmp; fstatus refnum
	while(V_tmp!=4)
	V_windowTitleLen=V_filePos-V_startWindowTitle-1

	S_comment=padstring("",V_commentLen,0x20)
	fsetpos refnum,V_startComment; fbinread refnum,S_comment
	S_comment=replacestring(num2char(10),S_comment,"")
	S_comment=replacestring(";",S_comment,"")

	S_windowTitle=padstring("",V_windowTitleLen,0x20)
	fsetpos refnum,V_startWindowTitle; fbinread refnum,S_windowTitle
	S_windowTitle=replacestring(";",S_windowTitle,"")

	close refnum

	sprintf noteStr, "identification:%s;window title:%s;comment:%s" S_identification, S_windowTitle, S_comment
	sprintf noteStr, "%s;version:%G;measure minium:%G;measure maximum:%G", noteStr, V_version, V_measureMin, V_measureMax
	sprintf noteStr, "%s;alarm minimum%G;alarm maximum:%G;alarm delay/min:%d" noteStr, V_alarmMin, V_alarmMax, V_alarmDelay
	sprintf noteStr, "%s;start time:%.0f;delta time:%u;" noteStr, V_timeStart, V_timeDelta
	sprintf noteStr, "<fileheader>%s</fileheader>" noteStr
	note w, noteStr

	if(V_version!=74&&V_version!=73)
		udFL.errCode = 0
		udFL.errMsg = "MiniLog version of file "+S_filenameLocal+".el1 is not 7.3 or 7.4 (with which this file loader has been tested).\rPlease check the data carefully for reliability!"
		udFLHandleError(udFL)
	endif
	
	return S_filenamelocal
end

// **********************************************
// Process the Waves

Static Function/S ProcessMiniSoftWaves(WavesLoaded,udFL)
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL

	variable ic, nt
	string theOne
	
	switch(udFL.userPCtrl)
		case 2:	// remove scaling
			nt = ItemsInList(WavesLoaded)
			for (ic=0;ic<nt;ic+=1)
				theOne = StringFromList(ic,WavesLoaded)
				setscale/p x 0,1,"" $theOne
				setscale d 0,0,"" $theOne
			endfor
			break
	endswitch
	
	return WavesLoaded

end

// **********************************************
// Apply User Data to Waves
//
// [...] wave selection

Static Function/S DoUserDataMiniSoftWaves(WavesLoaded, udFL)
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL	

	return WavesLoaded
end

// **********************************************
// Rename Waves

Static Function/S RenameMiniSoftWaves(ic,WavesLoaded,udFL)
	variable ic
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL

	string fileStr = StringFromList(ic,udFL.fileList)
		
	switch(udFL.setNames)
		case 1:
			WavesLoaded=udFLRenameInput(WavesLoaded,"MiniSoftWave")
			break
		case 2:
			udFL.namePrefix = ReplaceString(" ",ParseFilePath(3,fileStr,":",0,0),"")
			if (udFL.reportCtrl >= 1)
				print "... renaming wave(s) by filename ", udFL.namePrefix, " + number"
			endif
			WavesLoaded=udFLRenameInput(WavesLoaded,udFL.namePrefix)
			break
		case 8:
			if (udFL.reportCtrl >= 1)
				print "... renaming wave(s) by prefix ", udFL.namePrefix, " + number"
			endif
			WavesLoaded=udFLRenameInput(WavesLoaded,udFL.namePrefix)
			break
	endswitch
	return WavesLoaded
end

// **********************************************
// (possibly) Link the Waves

Static Function LinkMiniSoftWaves(WavesLoaded,udFL)
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL

	return 0
end